/*
 * Peony-Qt
 *
 * Copyright (C) 2024, Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * @author: Renyg <renyangguang@kylinos.cn>
 * @date:   2024-08-02 17:33
 */

#ifndef BLACKWHITELISTMANAGER_H
#define BLACKWHITELISTMANAGER_H

#include <QObject>
#include "singleton-utils.h"
#include <QDBusInterface>
#include <QSet>
#include "peony-core_global.h"

namespace Peony {

#define BW_LIST_NORMAL  "normal"
#define BW_LIST_BLACK   "blacklist"
#define BW_LIST_WHITE   "whitelist"

class FileLaunchAction;
class PEONYCORESHARED_EXPORT BlackWhiteListManager : public QObject
{
    Q_OBJECT
    DECLARESINGLETON(BlackWhiteListManager)

public:
    explicit BlackWhiteListManager(QObject *parent = nullptr);
    ~BlackWhiteListManager();

public:
    void setBWListModel(QString model);
    int addBWListElement(QString name);
    int clearBWlist();
    bool desktopNameExist(QString desktop);
    bool filter(const QString& desktopName) const;
    bool filterByAction(FileLaunchAction* action) const;
    void setDefaultActionStatus(FileLaunchAction* action) const;
    bool isAddDefaultAction(FileLaunchAction* action) const;

    bool isBlackListMode() const;
    bool isWhiteListMode() const;
    bool isNormalMode() const;

    QString getMode() const;
    QSet<QString> getBWListInfo() const;

public Q_SLOTS:
    void syncBlackAndWhiteData();

private:
    QSet<QString> m_setBwListInfo;
    QString m_strWorkModel = BW_LIST_NORMAL;
    QDBusInterface* mDbusPeonyServer = nullptr;
};

#define BlackWhiteListManagerInstance  SINGLETON(BlackWhiteListManager)
#define BlackWhiteListManagerFilter(name)  SINGLETON(BlackWhiteListManager)->filter(name)
#define BlackWhiteListManagerFilterByAction(action)  SINGLETON(BlackWhiteListManager)->filterByAction(action)
#define BlackWhiteListSetDefaultAction(action)  SINGLETON(BlackWhiteListManager)->setDefaultActionStatus(action)
#define BlackWhiteListIsAddDefaultAction(action)  SINGLETON(BlackWhiteListManager)->isAddDefaultAction(action)

}

//#define BlackWhiteListManagerInstance  SINGLETON(Peony::BlackWhiteListManager)
//#define BlackWhiteListManagerFilter(name)  SINGLETON(Peony::BlackWhiteListManager)->filter(name)
//#define BlackWhiteListManagerFilterByAction(action)  SINGLETON(Peony::BlackWhiteListManager)->filterByAction(action)
//#define BlackWhiteListSetDefaultAction(action)  SINGLETON(Peony::BlackWhiteListManager)->setDefaultActionStatus(action)
//#define BlackWhiteListIsAddDefaultAction(action)  SINGLETON(Peony::BlackWhiteListManager)->isAddDefaultAction(action)

#endif // BLACKWHITELISTMANAGER_H
