﻿/*
 * Peony-Qt
 *
 * Copyright (C) 2024, Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * @author: Renyg <renyangguang@kylinos.cn>
 * @date:   2024-08-02 18:03
 */
#ifndef _KYLIN_MACRO_UTILS_H_
#define _KYLIN_MACRO_UTILS_H_

//namespace Peony {

#define DefineInstance(INSTNACE)\
template<typename T>\
class INSTNACE\
{\
public:\
    static T& instance()\
    {\
        static T ret;\
        return ret;\
    }\
}\

DefineInstance(Instance);

#define Global(ClassName)\
struct __##ClassName##Struct\
{\
    __##ClassName##Struct();\
}__##ClassName##instance;\
__##ClassName##Struct::__##ClassName##Struct()


#include <QVariant>

#define CUSTOM_PROPERTY(T, Name) \
    Q_PROPERTY(T Name READ get##Name WRITE set##Name NOTIFY sig##Name##Changed) \
    public: \
    T get##Name()const { return _Custom_Prop_##Name.value<T>(); } \
    void set##Name(const T& val) { _Custom_Prop_##Name = val;  emit sig##Name##Changed(); } \
    Q_SIGNAL void sig##Name##Changed(); \
    private:\
    QVariant _Custom_Prop_##Name;

//! logic control
#define CheckAndReturn(T, ...) if(T){ return __VA_ARGS__; }
#define CheckAndReturnAssert(T, ...) if(T){ Q_ASSERT(false); return __VA_ARGS__; }
#define CheckAndContinue(T) if(T){ continue; }
#define CheckAndContinueAssert(T) if(T){ Q_ASSERT(false); continue; }
#define CheckAndBreak(T) if(T){ break; }

//! pointer Delete
#define DSAFE_DELETE(a) {if(nullptr!=(a)){delete (a); (a)=nullptr;}}
#define DSAFE_DELETE_LATER(a) {if(nullptr!=(a)){(a)->deleteLater(); (a)=nullptr;}}
#define DSAFE_DELETE_ALL(container) {qDeleteAll(container);container.clear();}

//}
#endif // _KYLIN_MACRO_UTILS_H_
